<?php

namespace Drupal\redirects_fixer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Redirect fixer settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'redirects_fixer_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['redirects_fixer.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['site-domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site domain'),
      '#default_value' => $this->config('redirects_fixer.settings')->get('site-domain'),
      '#description' => $this->t('Enter the site domain. This value will be used to determine links absolute internal links in content. Do not include http:// or https://.'),
      '#required' => TRUE,
    ];
    $form['items_per_batch_node'] = [
      '#type' => 'textfield',
      '#attributes' => array(
        ' type' => 'number',
      ),
      '#title' => $this->t('Items per batch node'),
      '#default_value' => $this->config('redirects_fixer.settings')->get('items_per_batch_node') ?? '50',
      '#description' => $this->t('Number of nodes per batch node.'),
    ];
    $form['items_per_batch_block_content'] = [
      '#type' => 'textfield',
      '#attributes' => array(
        ' type' => 'number',
      ),
      '#title' => $this->t('Items per batch node'),
      '#default_value' => $this->config('redirects_fixer.settings')->get('items_per_batch_block_content') ?? '20',
      '#description' => $this->t('Number of nodes per batch lock content.'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (str_starts_with($form_state->getValue('site-domain'), 'http')) {
      $form_state->setErrorByName('site-domain', $this->t('Site domain filed should not contain the protocol.'));
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('redirects_fixer.settings')
      ->set('site-domain', $form_state->getValue('site-domain'))
      ->set('items_per_batch_node', $form_state->getValue('items_per_batch_node'))
      ->set('items_per_batch_block_content', $form_state->getValue('items_per_batch_block_content'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
