<?php

namespace Drupal\redirects_fixer;

use Drupal\Component\Utility\Html;
use Drupal\Core\Database\Connection;

/**
 * Links finder.
 */
class LinksFinder {

  const DEBUG_NIDS = [
    '33112', // Node with fragment.
    '35520', // Problematic node where replacements do not happen.
  ];

  const DEBUG = FALSE;

  /**
   * The database connection to be used.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * Constructs a LinksFinder object.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Get total count fields with links.
   *
   * @todo: add description.
   */
  public function totalCountFields(string $table, string $column): int {

    $query = $this->database
      ->select($table, $table)
      ->condition( $table . '.' .  $column, '%'. $this->database->escapeLike('<a') . '%', 'LIKE');

    if (self::DEBUG) {
      $query->condition($table . '.' . 'entity_id',  self::DEBUG_NIDS, 'IN');
    }

    return $query->countQuery()->execute()->fetchField();
  }

  /**
   * Get fields.
   *
   * @todo Add description.
   */
  public function getLinksFields(string $table, string $field, int $offset, int $limit): array {
    $column = $field;

    $query = $this->database
      ->select($table, $table)
      ->fields($table, ['entity_id', $column])
      ->range($offset, $limit);

    if (self::DEBUG) {
      $query->condition($table . '.' . 'entity_id',  self::DEBUG_NIDS, 'IN');
    }

    return $query->execute()->fetchAllKeyed();
  }

  /**
   * Prepare links data for manipulations.
   */
  public function prepareLinksData(array $links_fields): array {
    foreach ($links_fields as &$field_value) {
      $document = Html::load($field_value);
      $field_value = [
        'document' => $document,
        'list' => $document->getElementsByTagName('a'),
      ];
    }
    return $links_fields;
  }

  /**
   * Update data in entity field.
   */
  public function updateLinksData(string $table, string $field, array $data, string $table_revision): void {
    foreach ($data as $id => $val) {
      $this->database->update($table)
        ->fields([$field => $val])
        ->condition('entity_id', $id)
        ->execute();

      $query = "SELECT MAX(revision_id) as max_revision FROM {$table_revision} WHERE entity_id = {$id} GROUP BY entity_id;";
      $max_revision_id = $this->database->query($query)->fetchField();

      $query = "SELECT revision_id FROM {$table} WHERE entity_id = {$id};";
      $current_revision_id = $this->database->query($query)->fetchField();

      $revision_ids = array_unique([$max_revision_id, $current_revision_id]);
      foreach ($revision_ids as $revision_id) {
        $this->database->update($table_revision)
          ->fields([$field => $val])
          ->condition('entity_id', $id)
          ->condition('revision_id', $revision_id)
          ->execute();
      }
    }
  }

}
